package ru.dbotthepony.mc.otm.registry

import net.minecraft.resources.ResourceLocation
import net.minecraft.world.item.crafting.Recipe
import net.minecraft.world.item.crafting.RecipeType
import net.minecraftforge.eventbus.api.IEventBus
import net.minecraftforge.registries.DeferredRegister
import net.minecraftforge.registries.ForgeRegistries
import net.minecraftforge.registries.RegistryObject
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.recipe.EnergyContainerRecipe
import ru.dbotthepony.mc.otm.recipe.ExplosiveHammerPrimingRecipe
import ru.dbotthepony.mc.otm.recipe.PlatePressRecipe
import ru.dbotthepony.mc.otm.recipe.UpgradeRecipe

@Suppress("SameParameterValue")
object MRecipes {
	class Type<T : Recipe<*>>(id: String) : RecipeType<T> {
		val id = ResourceLocation(OverdriveThatMatters.MOD_ID, id)

		override fun toString(): String {
			return id.toString()
		}
	}

	private val types = DeferredRegister.create(ForgeRegistries.RECIPE_TYPES, OverdriveThatMatters.MOD_ID)
	private val serializers = DeferredRegister.create(ForgeRegistries.RECIPE_SERIALIZERS, OverdriveThatMatters.MOD_ID)

	internal fun register(bus: IEventBus) {
		types.register(bus)
		serializers.register(bus)
	}

	private fun <T : Recipe<*>> register(name: String): RegistryObject<Type<T>> {
		return types.register(name) { Type(name) }
	}

	val PLATE_PRESS by register<PlatePressRecipe>("plate_press")

	init {
		serializers.register("plate_press") { PlatePressRecipe.SERIALIZER }
		serializers.register("energy_container") { EnergyContainerRecipe.Companion }
		serializers.register("upgrade") { UpgradeRecipe.Companion }
		serializers.register("hammer_priming") { ExplosiveHammerPrimingRecipe.Companion }
	}
}
